"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuroraServerlessMySQLDatabaseReader = void 0;
const RelationalDBSchemaTransformer_1 = require("./RelationalDBSchemaTransformer");
const RelationalDBSchemaTransformerUtils_1 = require("./RelationalDBSchemaTransformerUtils");
const AuroraDataAPIClient_1 = require("./AuroraDataAPIClient");
const graphql_transformer_common_1 = require("graphql-transformer-common");
class AuroraServerlessMySQLDatabaseReader {
    setAuroraClient(auroraClient) {
        this.auroraClient = auroraClient;
    }
    constructor(dbRegion, awsSecretStoreArn, dbClusterOrInstanceArn, database, aws) {
        this.hydrateTemplateContext = async (contextShell) => {
            contextShell.secretStoreArn = this.awsSecretStoreArn;
            contextShell.rdsClusterIdentifier = this.dbClusterOrInstanceArn;
            contextShell.databaseSchema = 'mysql';
            contextShell.databaseName = this.database;
            contextShell.region = this.dbRegion;
            return contextShell;
        };
        this.listTables = async () => {
            const results = await this.auroraClient.listTables();
            return results;
        };
        this.getTableForeignKeyReferences = async (tableName) => {
            const results = await this.auroraClient.getTableForeignKeyReferences(tableName);
            return results;
        };
        this.describeTable = async (tableName) => {
            const columnDescriptions = await this.auroraClient.describeTable(tableName);
            const fields = new Array();
            const updateFields = new Array();
            const createFields = new Array();
            let primaryKey = '';
            let primaryKeyType = '';
            const intFieldList = new Array();
            const stringFieldList = new Array();
            const formattedTableName = (0, graphql_transformer_common_1.toUpper)(tableName);
            for (const columnDescription of columnDescriptions) {
                if (columnDescription.Key == 'PRI') {
                    primaryKey = columnDescription.Field;
                    primaryKeyType = (0, RelationalDBSchemaTransformerUtils_1.getGraphQLTypeFromMySQLType)(columnDescription.Type);
                }
                else {
                    const type = (0, RelationalDBSchemaTransformerUtils_1.getGraphQLTypeFromMySQLType)(columnDescription.Type);
                    if (type === 'Int') {
                        intFieldList.push(columnDescription.Field);
                    }
                    else if (type === 'String') {
                        stringFieldList.push(columnDescription.Field);
                    }
                }
                const baseType = (0, RelationalDBSchemaTransformerUtils_1.getNamedType)((0, RelationalDBSchemaTransformerUtils_1.getGraphQLTypeFromMySQLType)(columnDescription.Type));
                const isPrimaryKey = columnDescription.Key == 'PRI';
                const isNullable = columnDescription.Null == 'YES';
                const type = !isPrimaryKey && isNullable ? baseType : (0, RelationalDBSchemaTransformerUtils_1.getNonNullType)(baseType);
                fields.push((0, RelationalDBSchemaTransformerUtils_1.getFieldDefinition)(columnDescription.Field, type));
                createFields.push((0, RelationalDBSchemaTransformerUtils_1.getInputValueDefinition)(type, columnDescription.Field));
                const updateType = !isPrimaryKey ? baseType : (0, RelationalDBSchemaTransformerUtils_1.getNonNullType)(baseType);
                updateFields.push((0, RelationalDBSchemaTransformerUtils_1.getInputValueDefinition)(updateType, columnDescription.Field));
            }
            return new RelationalDBSchemaTransformer_1.TableContext((0, RelationalDBSchemaTransformerUtils_1.getTypeDefinition)(fields, tableName), (0, RelationalDBSchemaTransformerUtils_1.getInputTypeDefinition)(createFields, `Create${formattedTableName}Input`), (0, RelationalDBSchemaTransformerUtils_1.getInputTypeDefinition)(updateFields, `Update${formattedTableName}Input`), primaryKey, primaryKeyType, stringFieldList, intFieldList);
        };
        this.auroraClient = new AuroraDataAPIClient_1.AuroraDataAPIClient(dbRegion, awsSecretStoreArn, dbClusterOrInstanceArn, database, aws);
        this.dbRegion = dbRegion;
        this.awsSecretStoreArn = awsSecretStoreArn;
        this.dbClusterOrInstanceArn = dbClusterOrInstanceArn;
        this.database = database;
    }
}
exports.AuroraServerlessMySQLDatabaseReader = AuroraServerlessMySQLDatabaseReader;
//# sourceMappingURL=AuroraServerlessMySQLDatabaseReader.js.map