"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RelationalDBResolverGenerator = void 0;
const fs = __importStar(require("fs-extra"));
const graphql_mapping_template_1 = require("graphql-mapping-template");
const graphql_transformer_common_1 = require("graphql-transformer-common");
const appSync_1 = __importDefault(require("cloudform-types/types/appSync"));
const cloudform_types_1 = require("cloudform-types");
const RelationalDBMappingTemplate_1 = require("./RelationalDBMappingTemplate");
const ResourceConstants_1 = require("./ResourceConstants");
const s3BaseUrl = 's3://${S3DeploymentBucket}/${S3DeploymentRootKey}/resolvers/${ResolverFileName}';
const resolverFileName = 'ResolverFileName';
const rdsResponseErrorMessage = 'Invalid response from RDS DataSource. See info for the full response.';
const rdsResponseErrorType = 'InvalidResponse';
class RelationalDBResolverGenerator {
    constructor(context) {
        this.document = context.schemaDoc;
        this.typePrimaryKeyMap = context.typePrimaryKeyMap;
        this.stringFieldMap = context.stringFieldMap;
        this.intFieldMap = context.intFieldMap;
        this.typePrimaryKeyTypeMap = context.typePrimaryKeyTypeMap;
        this.variableMapRefName = 'variableMap';
    }
    createRelationalResolvers(resolverFilePath, improvePluralization) {
        let resources = {};
        this.resolverFilePath = resolverFilePath;
        this.typePrimaryKeyMap.forEach((value, key) => {
            const resourceName = key.replace(/[^A-Za-z0-9]/g, '');
            resources = {
                ...resources,
                ...{ [resourceName + 'CreateResolver']: this.makeCreateRelationalResolver(key) },
                ...{ [resourceName + 'GetResolver']: this.makeGetRelationalResolver(key) },
                ...{ [resourceName + 'UpdateResolver']: this.makeUpdateRelationalResolver(key) },
                ...{ [resourceName + 'DeleteResolver']: this.makeDeleteRelationalResolver(key) },
                ...{ [resourceName + 'ListResolver']: this.makeListRelationalResolver(key, improvePluralization) },
            };
        });
        return resources;
    }
    makeCreateRelationalResolver(type, mutationTypeName = 'Mutation') {
        const tableName = this.getTableName(type);
        const operationType = GRAPHQL_RESOLVER_OPERATION.Create;
        const fieldName = this.getFieldName(type, operationType);
        const createSql = this.generateInsertStatement(type);
        const selectSql = this.generateSelectByPrimaryKeyStatement(type, operationType);
        const reqFileName = `${mutationTypeName}.${fieldName}.req.vtl`;
        const resFileName = `${mutationTypeName}.${fieldName}.res.vtl`;
        const reqTemplate = (0, graphql_mapping_template_1.print)((0, graphql_mapping_template_1.compoundExpression)([
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('cols'), (0, graphql_mapping_template_1.list)([])),
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('vals'), (0, graphql_mapping_template_1.list)([])),
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)(this.variableMapRefName), (0, graphql_mapping_template_1.obj)({})),
            (0, graphql_mapping_template_1.forEach)((0, graphql_mapping_template_1.ref)('entry'), (0, graphql_mapping_template_1.ref)(`ctx.args.create${tableName}Input.keySet()`), [
                (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('discard'), (0, graphql_mapping_template_1.ref)(`cols.add($entry)`)),
                (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('discard'), (0, graphql_mapping_template_1.ref)(`vals.add(":$entry")`)),
                (0, graphql_mapping_template_1.methodCall)((0, graphql_mapping_template_1.ref)('util.qr'), (0, graphql_mapping_template_1.methodCall)((0, graphql_mapping_template_1.ref)(`${this.variableMapRefName}.put`), (0, graphql_mapping_template_1.str)(':$entry'), (0, graphql_mapping_template_1.ref)(`ctx.args.create${tableName}Input[$entry]`))),
            ]),
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('valStr'), (0, graphql_mapping_template_1.ref)('vals.toString().replace("[","(").replace("]",")")')),
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('colStr'), (0, graphql_mapping_template_1.ref)('cols.toString().replace("[","(").replace("]",")")')),
            RelationalDBMappingTemplate_1.RelationalDBMappingTemplate.rdsQuery({
                statements: (0, graphql_mapping_template_1.list)([(0, graphql_mapping_template_1.str)(createSql), (0, graphql_mapping_template_1.str)(selectSql)]),
                variableMapRefName: this.variableMapRefName
            }),
        ]));
        const resTemplate = (0, graphql_mapping_template_1.print)((0, graphql_mapping_template_1.ref)('utils.toJson($utils.parseJson($utils.rds.toJsonString($ctx.result))[1][0])'));
        fs.writeFileSync(`${this.resolverFilePath}/${reqFileName}`, reqTemplate, 'utf8');
        fs.writeFileSync(`${this.resolverFilePath}/${resFileName}`, resTemplate, 'utf8');
        let resolver = new appSync_1.default.Resolver({
            ApiId: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.AppSyncApiId),
            DataSourceName: cloudform_types_1.Fn.GetAtt(ResourceConstants_1.ResourceConstants.RESOURCES.RelationalDatabaseDataSource, 'Name'),
            TypeName: mutationTypeName,
            FieldName: fieldName,
            RequestMappingTemplateS3Location: cloudform_types_1.Fn.Sub(s3BaseUrl, {
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket),
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey),
                [resolverFileName]: reqFileName,
            }),
            ResponseMappingTemplateS3Location: cloudform_types_1.Fn.Sub(s3BaseUrl, {
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket),
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey),
                [resolverFileName]: resFileName,
            }),
        }).dependsOn([ResourceConstants_1.ResourceConstants.RESOURCES.RelationalDatabaseDataSource]);
        return resolver;
    }
    makeGetRelationalResolver(type, queryTypeName = 'Query') {
        const operationType = GRAPHQL_RESOLVER_OPERATION.Get;
        const fieldName = this.getFieldName(type, operationType);
        const selectSql = this.generateSelectByPrimaryKeyStatement(type, operationType);
        const reqFileName = `${queryTypeName}.${fieldName}.req.vtl`;
        const resFileName = `${queryTypeName}.${fieldName}.res.vtl`;
        const primaryKey = this.getTablePrimaryKey(type);
        const primaryKeyRef = this.getPrimaryKeyRef(type, operationType);
        const reqTemplate = (0, graphql_mapping_template_1.print)((0, graphql_mapping_template_1.compoundExpression)([
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)(this.variableMapRefName), (0, graphql_mapping_template_1.obj)({})),
            (0, graphql_mapping_template_1.methodCall)((0, graphql_mapping_template_1.ref)('util.qr'), (0, graphql_mapping_template_1.methodCall)((0, graphql_mapping_template_1.ref)(`${this.variableMapRefName}.put`), (0, graphql_mapping_template_1.str)(`:${primaryKey}`), primaryKeyRef)),
            RelationalDBMappingTemplate_1.RelationalDBMappingTemplate.rdsQuery({
                statements: (0, graphql_mapping_template_1.list)([(0, graphql_mapping_template_1.str)(selectSql)]),
                variableMapRefName: this.variableMapRefName
            }),
        ]));
        const resTemplate = (0, graphql_mapping_template_1.print)((0, graphql_mapping_template_1.compoundExpression)([
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('output'), (0, graphql_mapping_template_1.ref)('utils.rds.toJsonObject($ctx.result)')),
            (0, graphql_mapping_template_1.iff)((0, graphql_mapping_template_1.ref)('output.isEmpty()'), (0, graphql_mapping_template_1.methodCall)((0, graphql_mapping_template_1.ref)('util.error'), (0, graphql_mapping_template_1.str)(rdsResponseErrorMessage), (0, graphql_mapping_template_1.str)(rdsResponseErrorType), (0, graphql_mapping_template_1.obj)({}), (0, graphql_mapping_template_1.ref)('output'))),
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('output'), (0, graphql_mapping_template_1.ref)('output[0]')),
            (0, graphql_mapping_template_1.iff)((0, graphql_mapping_template_1.ref)('output.isEmpty()'), (0, graphql_mapping_template_1.ret)()),
            (0, graphql_mapping_template_1.methodCall)((0, graphql_mapping_template_1.ref)('utils.toJson'), (0, graphql_mapping_template_1.ref)('output[0]')),
        ]));
        fs.writeFileSync(`${this.resolverFilePath}/${reqFileName}`, reqTemplate, 'utf8');
        fs.writeFileSync(`${this.resolverFilePath}/${resFileName}`, resTemplate, 'utf8');
        let resolver = new appSync_1.default.Resolver({
            ApiId: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.AppSyncApiId),
            DataSourceName: cloudform_types_1.Fn.GetAtt(ResourceConstants_1.ResourceConstants.RESOURCES.RelationalDatabaseDataSource, 'Name'),
            FieldName: fieldName,
            TypeName: queryTypeName,
            RequestMappingTemplateS3Location: cloudform_types_1.Fn.Sub(s3BaseUrl, {
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket),
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey),
                [resolverFileName]: reqFileName,
            }),
            ResponseMappingTemplateS3Location: cloudform_types_1.Fn.Sub(s3BaseUrl, {
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket),
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey),
                [resolverFileName]: resFileName,
            }),
        }).dependsOn([ResourceConstants_1.ResourceConstants.RESOURCES.RelationalDatabaseDataSource]);
        return resolver;
    }
    makeUpdateRelationalResolver(type, mutationTypeName = 'Mutation') {
        const tableName = this.getTableName(type);
        const operationType = GRAPHQL_RESOLVER_OPERATION.Update;
        const fieldName = this.getFieldName(type, operationType);
        const updateSql = this.generateUpdateStatement(type);
        const selectSql = this.generateSelectByPrimaryKeyStatement(type, operationType);
        const reqFileName = `${mutationTypeName}.${fieldName}.req.vtl`;
        const resFileName = `${mutationTypeName}.${fieldName}.res.vtl`;
        const reqTemplate = (0, graphql_mapping_template_1.print)((0, graphql_mapping_template_1.compoundExpression)([
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('updateList'), (0, graphql_mapping_template_1.obj)({})),
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)(this.variableMapRefName), (0, graphql_mapping_template_1.obj)({})),
            (0, graphql_mapping_template_1.forEach)((0, graphql_mapping_template_1.ref)('entry'), (0, graphql_mapping_template_1.ref)(`ctx.args.update${tableName}Input.keySet()`), [
                (0, graphql_mapping_template_1.methodCall)((0, graphql_mapping_template_1.ref)('util.qr'), (0, graphql_mapping_template_1.methodCall)((0, graphql_mapping_template_1.ref)(`${this.variableMapRefName}.put`), (0, graphql_mapping_template_1.str)(':$entry'), (0, graphql_mapping_template_1.ref)(`ctx.args.update${tableName}Input[$entry]`))),
                (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('discard'), (0, graphql_mapping_template_1.ref)(`updateList.put($entry, ":$entry")`)),
            ]),
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('update'), (0, graphql_mapping_template_1.ref)(`updateList.toString().replace("{","").replace("}","")`)),
            RelationalDBMappingTemplate_1.RelationalDBMappingTemplate.rdsQuery({
                statements: (0, graphql_mapping_template_1.list)([(0, graphql_mapping_template_1.str)(updateSql), (0, graphql_mapping_template_1.str)(selectSql)]),
                variableMapRefName: this.variableMapRefName
            }),
        ]));
        const resTemplate = (0, graphql_mapping_template_1.print)((0, graphql_mapping_template_1.compoundExpression)([
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('output'), (0, graphql_mapping_template_1.ref)('utils.rds.toJsonObject($ctx.result)')),
            (0, graphql_mapping_template_1.iff)((0, graphql_mapping_template_1.ref)('output.length() < 2'), (0, graphql_mapping_template_1.methodCall)((0, graphql_mapping_template_1.ref)('util.error'), (0, graphql_mapping_template_1.str)(rdsResponseErrorMessage), (0, graphql_mapping_template_1.str)(rdsResponseErrorType), (0, graphql_mapping_template_1.obj)({}), (0, graphql_mapping_template_1.ref)('output'))),
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('output'), (0, graphql_mapping_template_1.ref)('output[1]')),
            (0, graphql_mapping_template_1.iff)((0, graphql_mapping_template_1.ref)('output.isEmpty()'), (0, graphql_mapping_template_1.ret)()),
            (0, graphql_mapping_template_1.methodCall)((0, graphql_mapping_template_1.ref)('utils.toJson'), (0, graphql_mapping_template_1.ref)('output[0]')),
        ]));
        fs.writeFileSync(`${this.resolverFilePath}/${reqFileName}`, reqTemplate, 'utf8');
        fs.writeFileSync(`${this.resolverFilePath}/${resFileName}`, resTemplate, 'utf8');
        let resolver = new appSync_1.default.Resolver({
            ApiId: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.AppSyncApiId),
            DataSourceName: cloudform_types_1.Fn.GetAtt(ResourceConstants_1.ResourceConstants.RESOURCES.RelationalDatabaseDataSource, 'Name'),
            TypeName: mutationTypeName,
            FieldName: fieldName,
            RequestMappingTemplateS3Location: cloudform_types_1.Fn.Sub(s3BaseUrl, {
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket),
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey),
                [resolverFileName]: reqFileName,
            }),
            ResponseMappingTemplateS3Location: cloudform_types_1.Fn.Sub(s3BaseUrl, {
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket),
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey),
                [resolverFileName]: resFileName,
            }),
        }).dependsOn([ResourceConstants_1.ResourceConstants.RESOURCES.RelationalDatabaseDataSource]);
        return resolver;
    }
    makeDeleteRelationalResolver(type, mutationTypeName = 'Mutation') {
        const operationType = GRAPHQL_RESOLVER_OPERATION.Delete;
        const fieldName = this.getFieldName(type, operationType);
        const selectSql = this.generateSelectByPrimaryKeyStatement(type, operationType);
        const deleteSql = this.generateDeleteStatement(type);
        const reqFileName = `${mutationTypeName}.${fieldName}.req.vtl`;
        const resFileName = `${mutationTypeName}.${fieldName}.res.vtl`;
        const primaryKey = this.getTablePrimaryKey(type);
        const primaryKeyRef = this.getPrimaryKeyRef(type, operationType);
        const reqTemplate = (0, graphql_mapping_template_1.print)((0, graphql_mapping_template_1.compoundExpression)([
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)(this.variableMapRefName), (0, graphql_mapping_template_1.obj)({})),
            (0, graphql_mapping_template_1.methodCall)((0, graphql_mapping_template_1.ref)('util.qr'), (0, graphql_mapping_template_1.methodCall)((0, graphql_mapping_template_1.ref)(`${this.variableMapRefName}.put`), (0, graphql_mapping_template_1.str)(`:${primaryKey}`), primaryKeyRef)),
            RelationalDBMappingTemplate_1.RelationalDBMappingTemplate.rdsQuery({
                statements: (0, graphql_mapping_template_1.list)([(0, graphql_mapping_template_1.str)(selectSql), (0, graphql_mapping_template_1.str)(deleteSql)]),
                variableMapRefName: this.variableMapRefName
            }),
        ]));
        const resTemplate = (0, graphql_mapping_template_1.print)((0, graphql_mapping_template_1.compoundExpression)([
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('output'), (0, graphql_mapping_template_1.ref)('utils.rds.toJsonObject($ctx.result)')),
            (0, graphql_mapping_template_1.iff)((0, graphql_mapping_template_1.ref)('output.isEmpty()'), (0, graphql_mapping_template_1.methodCall)((0, graphql_mapping_template_1.ref)('util.error'), (0, graphql_mapping_template_1.str)(rdsResponseErrorMessage), (0, graphql_mapping_template_1.str)(rdsResponseErrorType), (0, graphql_mapping_template_1.obj)({}), (0, graphql_mapping_template_1.ref)('output'))),
            (0, graphql_mapping_template_1.set)((0, graphql_mapping_template_1.ref)('output'), (0, graphql_mapping_template_1.ref)('output[0]')),
            (0, graphql_mapping_template_1.iff)((0, graphql_mapping_template_1.ref)('output.isEmpty()'), (0, graphql_mapping_template_1.ret)()),
            (0, graphql_mapping_template_1.methodCall)((0, graphql_mapping_template_1.ref)('utils.toJson'), (0, graphql_mapping_template_1.ref)('output[0]')),
        ]));
        fs.writeFileSync(`${this.resolverFilePath}/${reqFileName}`, reqTemplate, 'utf8');
        fs.writeFileSync(`${this.resolverFilePath}/${resFileName}`, resTemplate, 'utf8');
        let resolver = new appSync_1.default.Resolver({
            ApiId: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.AppSyncApiId),
            DataSourceName: cloudform_types_1.Fn.GetAtt(ResourceConstants_1.ResourceConstants.RESOURCES.RelationalDatabaseDataSource, 'Name'),
            TypeName: mutationTypeName,
            FieldName: fieldName,
            RequestMappingTemplateS3Location: cloudform_types_1.Fn.Sub(s3BaseUrl, {
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket),
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey),
                [resolverFileName]: reqFileName,
            }),
            ResponseMappingTemplateS3Location: cloudform_types_1.Fn.Sub(s3BaseUrl, {
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket),
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey),
                [resolverFileName]: resFileName,
            }),
        }).dependsOn([ResourceConstants_1.ResourceConstants.RESOURCES.RelationalDatabaseDataSource]);
        return resolver;
    }
    makeListRelationalResolver(type, improvePluralization, queryTypeName = 'Query') {
        const fieldName = (0, graphql_transformer_common_1.graphqlName)(GRAPHQL_RESOLVER_OPERATION.List + (0, graphql_transformer_common_1.plurality)((0, graphql_transformer_common_1.toUpper)(type), improvePluralization));
        const selectSql = this.generateSelectStatement(type);
        const reqFileName = `${queryTypeName}.${fieldName}.req.vtl`;
        const resFileName = `${queryTypeName}.${fieldName}.res.vtl`;
        const reqTemplate = (0, graphql_mapping_template_1.print)(RelationalDBMappingTemplate_1.RelationalDBMappingTemplate.rdsQuery({
            statements: (0, graphql_mapping_template_1.list)([(0, graphql_mapping_template_1.str)(selectSql)]),
        }));
        const resTemplate = (0, graphql_mapping_template_1.print)((0, graphql_mapping_template_1.ref)('utils.toJson($utils.rds.toJsonObject($ctx.result)[0])'));
        fs.writeFileSync(`${this.resolverFilePath}/${reqFileName}`, reqTemplate, 'utf8');
        fs.writeFileSync(`${this.resolverFilePath}/${resFileName}`, resTemplate, 'utf8');
        let resolver = new appSync_1.default.Resolver({
            ApiId: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.AppSyncApiId),
            DataSourceName: cloudform_types_1.Fn.GetAtt(ResourceConstants_1.ResourceConstants.RESOURCES.RelationalDatabaseDataSource, 'Name'),
            TypeName: queryTypeName,
            FieldName: fieldName,
            RequestMappingTemplateS3Location: cloudform_types_1.Fn.Sub(s3BaseUrl, {
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket),
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey),
                [resolverFileName]: reqFileName,
            }),
            ResponseMappingTemplateS3Location: cloudform_types_1.Fn.Sub(s3BaseUrl, {
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentBucket),
                [ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey]: cloudform_types_1.Fn.Ref(ResourceConstants_1.ResourceConstants.PARAMETERS.S3DeploymentRootKey),
                [resolverFileName]: resFileName,
            }),
        }).dependsOn([ResourceConstants_1.ResourceConstants.RESOURCES.RelationalDatabaseDataSource]);
        return resolver;
    }
    getTableName(type) {
        return (0, graphql_transformer_common_1.toUpper)(type);
    }
    getFieldName(type, operationType) {
        const tableName = this.getTableName(type);
        return (0, graphql_transformer_common_1.graphqlName)(`${operationType}${tableName}`);
    }
    getTablePrimaryKey(type) {
        return this.typePrimaryKeyMap.get(type);
    }
    isPrimaryKeyAStringType(type) {
        return this.typePrimaryKeyTypeMap.get(type).includes('String');
    }
    generateSelectStatement(type) {
        const tableName = this.getTableName(type);
        return `SELECT * FROM ${tableName}`;
    }
    generateSelectByPrimaryKeyStatement(type, operationType) {
        const tableName = this.getTableName(type);
        const primaryKey = this.getTablePrimaryKey(type);
        return `SELECT * FROM ${tableName} WHERE ${primaryKey}=:${primaryKey}`;
    }
    getPrimaryKeyRef(type, operationType) {
        const tableName = this.getTableName(type);
        const primaryKey = this.getTablePrimaryKey(type);
        const hasToAppendOperationInput = ![GRAPHQL_RESOLVER_OPERATION.Get, GRAPHQL_RESOLVER_OPERATION.Delete].includes(operationType);
        const operationInput = hasToAppendOperationInput ? `${operationType}${tableName}Input.` : '';
        if (this.isPrimaryKeyAStringType(type)) {
            return (0, graphql_mapping_template_1.str)(`$ctx.args.${operationInput}${primaryKey}`);
        }
        return (0, graphql_mapping_template_1.ref)(`ctx.args.${operationInput}${primaryKey}`);
    }
    generateInsertStatement(type) {
        const tableName = this.getTableName(type);
        return `INSERT INTO ${tableName} $colStr VALUES $valStr`;
    }
    generateUpdateStatement(type) {
        const primaryKey = this.getTablePrimaryKey(type);
        return `UPDATE ${type} SET $update WHERE ${primaryKey}=:${primaryKey}`;
    }
    generateDeleteStatement(type) {
        const primaryKey = this.getTablePrimaryKey(type);
        return `DELETE FROM ${type} WHERE ${primaryKey}=:${primaryKey}`;
    }
}
exports.RelationalDBResolverGenerator = RelationalDBResolverGenerator;
var GRAPHQL_RESOLVER_OPERATION;
(function (GRAPHQL_RESOLVER_OPERATION) {
    GRAPHQL_RESOLVER_OPERATION["Create"] = "create";
    GRAPHQL_RESOLVER_OPERATION["Delete"] = "delete";
    GRAPHQL_RESOLVER_OPERATION["Get"] = "get";
    GRAPHQL_RESOLVER_OPERATION["List"] = "list";
    GRAPHQL_RESOLVER_OPERATION["Update"] = "update";
})(GRAPHQL_RESOLVER_OPERATION || (GRAPHQL_RESOLVER_OPERATION = {}));
//# sourceMappingURL=RelationalDBResolverGenerator.js.map